<?php

namespace Modules\St\Http\Controllers;

use Illuminate\Contracts\Support\Renderable;
use Illuminate\Http\Request;
use Illuminate\Routing\Controller;
use Illuminate\Support\Facades\DB;
use Modules\St\Entities\GLPosting;
use Modules\St\Entities\Suppliers;
use Modules\St\Entities\SupplierContact;
use Modules\St\Entities\Country;
use Modules\St\Entities\SupplierType;
use Modules\St\Entities\SupplierStatus;
use Modules\St\Entities\District;
use Modules\St\Entities\Town;
use Modules\St\Entities\Bank;
use Modules\St\Entities\BankBranch;

class SuppliersController extends Controller
{
    // This function is use to get town data of selected district
    function GetSelectedDistrictToTown($id) {
        try{$result = DB::table('st_towns')->select('town_id', 'town')->where('district_foreign_id', '=', $id)->get();

            $responseBody = $this->responseBody(true, "town", "Found", $result);
        } catch(\Exception $exception) {
            $responseBody = $this->responseBody(false, "town", "Error", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }

    // This function is use to get all GL Posting data
    function AllGLPostData() {
        try{$result = GLPosting::all();

            $responseBody = $this->responseBody(true, "GLPostData", "Found", $result);
        } catch(\Exception $exception) {
            $responseBody = $this->responseBody(false, "GLPostData", "Error", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }

    // This function is use to save a supplier
    function SaveSupplier(Request $request) {
        try{$supplier = new Suppliers();
            $supplier_contact = new SupplierContact();

            $supplier->supplier_id = $request->txtSupplierId;
            $supplier->supplier_name = $request->txtSupplierName;
            $supplier->address = $request->txtAddress;
            $supplier->web_address = $request->txtWebAddress;
            $supplier->gmap = $request->txtMapUrl;
            $supplier->tax1_no = $request->txtTax1_No;
            $supplier->tax2_no = $request->txtTax2_No;
            $supplier->alert_credit_limit = $request->txtAlertCreditLimit;
            $supplier->alert_credit_period = $request->txtAlertCreditPeriod;
            $supplier->hold_credit_limit = $request->txtHoldCreditLimit;
            $supplier->hold_credit_period = $request->txtHoldCreditPeriod;
            $supplier->notes = $request->textNotes;

            $supplier->country_id = $request->selectCountry;
            $supplier->district_id = $request->selectDistrict;
            $supplier->town_id = $request->selectTown;
            $supplier->supplier_type_id = $request->selectSupplierType;
            $supplier->account_group_id = $request->selectAccountgroup_id;
            $supplier->status_id = $request->selectSupplierStatus;
            $supplier->bank_id = $request->selectBank;
            $supplier->bank_branch_id = $request->selectBankBranch;

            $saveSupplier = $supplier->save();


            $supplier_contact->supplier_foriegn_id = $request->txtSupplierId;
            $supplier_contact->designation = $request->txtDesignation;
            $supplier_contact->email = $request->txtEmail;
            $supplier_contact->mobile = $request->txtMobile;
            $supplier_contact->fixed = $request->txtFixedMobile;

            $supplier_contact->sms_alert = $this->isChecked($request->checkSMSAlert);
            $supplier_contact->email_alert = $this->isChecked($request->checkEmailAlert);
            $supplier_contact->primary_contact = $this->isChecked($request->checkPrimaryContact);

            $saveSupplierContact = $supplier_contact->save();

            if($saveSupplier & $saveSupplierContact) {
                $responseBody = $this->responseBody(true, "supplier", "Saved", null);
            }
        } catch(\Exception $exception) {
                $responseBody = $this->responseBody(false, "Supplier", "Error", $exception);
        }
        return response()->json([ "data" => $responseBody ]);
    }

    // This function can be use to check the checkbox values and return boolean type value
    function isChecked($value) {
        if ($value == 'on') {
            return true;
        } else{
            return false;
        }
    }

    // This function is use to get All supplier data
    function AllSuppliers() {
        try{$result = Suppliers::all();

            $responseBody = $this->responseBody(true, "suppliers", "Found", $result);
        } catch(\Exception $exception) {
            $responseBody = $this->responseBody(false, "suppliers", "Error", $exception );
        }
        return response()->json(["data" => $responseBody]);
    }

    // View Supplier
    function Suppliers($id) {
        try{
            $supplier = Suppliers::find($id);
            $supplier_contact = DB::table('st_supplier_contacts')->where('supplier_foriegn_id', '=', $id)->first();

            $supplier_id = $supplier->supplier_id;
            $supplier_name = $supplier->supplier_name;
            $address = $supplier->address;
            $web_address = $supplier->web_address;
            $gmap = $supplier->gmap;
            $tax1_no = $supplier->tax1_no;
            $tax2_no = $supplier->tax2_no;
            $alert_credit_limit = $supplier->alert_credit_limit;
            $alert_credit_period = $supplier->alert_credit_period;
            $hold_credit_limit = $supplier->hold_credit_limit;
            $hold_credit_period = $supplier->hold_credit_period;
            $notes = $supplier->notes;

            $country = $this->GetNamesByID('country', $supplier->country_id);
            $district = $this->GetNamesByID('district', $supplier->district_id);
            $town = $this->GetNamesByID('town', $supplier->town_id);
            $supplier_type = $this->GetNamesByID('supplier_type', $supplier->supplier_type_id);
            $GLPost = $this->GetNamesByID('glposting', 1);
            $supplier_status = $this->GetNamesByID('supplier_status', $supplier->status_id);
            $bank = $this->GetNamesByID('bank', $supplier->bank_id);
            $bank_branch = $this->GetNamesByID('bankBranch', $supplier->bank_branch_id);

            $supplier_data=[];
            $supplier_data = [
                "supplier_id" => $supplier_id,
                "supplier_name" => $supplier_name,
                "address" => $address,
                "web_address" => $web_address,
                "gmap" => $gmap,
                "tax1_no" => $tax1_no,
                "tax2_no" => $tax2_no,
                "alert_credit_limit" => $alert_credit_limit,
                "alert_credit_period" => $alert_credit_period,
                "hold_credit_limit" => $hold_credit_limit,
                "hold_credit_period" => $hold_credit_period,
                "notes" => $notes,
                "country" => $country->country,
                "supplier_type" => $supplier_type->supplier_type, //error in this
                "GL_Post" => $GLPost->gl_post,
                "supplier_status" => $supplier_status->supplier_status,
                "district" => $district->district,
                "town" => $town->town,
                "bank" => $bank->bank_name,
                "bank_branch" => $bank_branch->bank_branch_name,
            ];

            $supplier_contact_data=[];
            $supplier_contact_data = [
               "contact_id" => $supplier_contact->contact_id,//error in this
               "email" => $supplier_contact->email,
               "designation" => $supplier_contact->designation,
               "mobile" => $supplier_contact->mobile,
               "fixed" => $supplier_contact->fixed,
               "primary_contact" => $supplier_contact->primary_contact,
               "sms_alert" => $supplier_contact->sms_alert,
               "email_alert" => $supplier_contact->email_alert,
            ];

            $responseBody = $this->responseBody(true, "suppliers", "Found", ["supplier" => $supplier_data, "supplier_contact" => $supplier_contact_data]);


        } catch(\Exception $exception) {
            $responseBody = $this->responseBody(false, "suppliers", "Error",  $exception);
        }
        return response()->json([ "data" => $responseBody]);
    }

    // Get Names of data using id
    function GetNamesByID($name, $id) {
        if ($name == 'country') {
            return Country::find($id);
        } else if ($name == 'supplier_type') {
            return SupplierType::find($id);
        } else if ($name == 'glposting') {
            return GLPosting::find($id);
        } else if ($name == 'supplier_status') {
            return SupplierStatus::find($id);
        } else if ($name == 'district') {
            return District::find($id);
        } else if ($name == 'town') {
            return Town::find($id);
        } else if ($name == 'bank') {
            return Bank::find($id);
        } else if ($name == 'bankBranch') {
            return BankBranch::find($id);
        }
    }

    // This function is use to update a supplier
    function UpdateSupplier(Request $request) {
        try{$id = $request->get('hiddenSupllierId');

            $supplier = Suppliers::find($id);

            $supplier->supplier_id = $request->txtSupplierId;
            $supplier->supplier_name = $request->txtSupplierName;
            $supplier->address = $request->txtAddress;
            $supplier->web_address = $request->txtWebAddress;
            $supplier->country_id = $request->selectCountry;
            $supplier->district_id = $request->selectDistrict;
            $supplier->town_id = $request->selectTown;
            $supplier->gmap = $request->txtMapUrl;
            $supplier->supplier_type_id = $request->selectSupplierType;
            $supplier->account_group_id = $request->selectAccountgroup_id;
            $supplier->bank_id = $request->selectBank;
            $supplier->bank_branch_id = $request->selectBankBranch;
            $supplier->tax1_no = $request->txtTax1_No;
            $supplier->tax2_no = $request->txtTax2_No;
            $supplier->alert_credit_limit = $request->txtAlertCreditLimit;
            $supplier->alert_credit_period = $request->txtAlertCreditPeriod;
            $supplier->hold_credit_limit = $request->txtHoldCreditLimit;
            $supplier->hold_credit_period = $request->txtHoldCreditPeriod;
            $supplier->status_id = $request->selectSupplierStatus;
            $supplier->notes = $request->textNotes;

            $updateSupplier = $supplier->save();


            $contact_id = $request->get('hiddenContactID');
            $supplier_contact = SupplierContact::find($contact_id);

            $supplier_contact->email = $request->txtEmail;
            $supplier_contact->designation = $request->txtDesignation;
            $supplier_contact->mobile = $request->txtMobile;
            $supplier_contact->fixed = $request->txtFixedMobile;
            $supplier_contact->sms_alert = $this->isChecked($request->checkSMSAlert);
            $supplier_contact->email_alert = $this->isChecked($request->checkEmailAlert);
            $supplier_contact->primary_contact = $this->isChecked($request->checkPrimaryContact);

            $updateSupplierContact = $supplier_contact->save();

            if ($updateSupplier & $updateSupplierContact) {
                $responseBody = $this->responseBody(true, "supplier", "updated", null);
            }
        } catch (\Exception $exception) {
                $responseBody = $this->responseBody(false, "supplier", "Error", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }

    // Delete a supplier
    function DeleteSupplier($id) {
        try{$object = Suppliers::find($id);
            $deleteSupplier = $object->delete();

           // $supplier_contact = DB::table('supplier_contacts')->where('supplier_foriegn_id', '=', $id)->first();
           $supplier_contact = SupplierContact::where('supplier_foriegn_id', '=', $id);
            $deleteSupplierContact = $supplier_contact->delete();

            if ($deleteSupplier & $deleteSupplierContact) {
                $responseBody = $this->responseBody(true, "supplier", "Deleted", null);
            }
        } catch(\Exception $exception) {
                $responseBody = $this->responseBody(false, "supplier", "Error", $exception);
        }
        return response()->json([ "data" => $responseBody]);
    }

    // Response Body function
    function responseBody($success, $name, $message, $result) {
        $body = [
            "success" => $success,
            "name" => $name,
            "message" => $message,
            "result" => $result
        ];
        return $body;
    }
}
